@@ -7,7 +7,7 @@ import time
 
 import skrt.core
 from skrt.image import Image
-from skrt.structures import RtStruct
+from skrt.structures import StructureSet
 
 
 class Study(skrt.core.Archive):
@@ -16,29 +16,33 @@ class Study(skrt.core.Archive):
         skrt.core.Archive.__init__(self, path, allow_dirs=True)
 
         special_dirs = ["RTPLAN", "RTSTRUCT", "RTDOSE"]
-        self.im_types = []
+        self.image_types = []
         for file in self.files:
 
             subdir = os.path.basename(file.path)
             if subdir in special_dirs:
                 continue
-            self.im_types.append(subdir)
+            self.image_types.append(subdir)
 
             # Get images
-            im_name = f"{subdir.lower()}_scans"
+            im_name = f"{subdir.lower()}_images"
             setattr(
                 self,
                 im_name,
                 self.get_dated_objects(dtype=Image, subdir=subdir, load=False),
             )
 
-            # Get associated structs
+            # Assign the original subdir name to the images
+            for im in getattr(self, im_name):
+                im.image_type = subdir
+
+            # Get associated structure sets
             struct_subdir = f"RTSTRUCT/{subdir}"
             if os.path.exists(os.path.join(self.path, struct_subdir)):
                 setattr(
                     self,
-                    f"{subdir.lower()}_structs",
-                    self.get_structs(
+                    f"{subdir.lower()}_structure_sets",
+                    self.get_structure_sets(
                         subdir=struct_subdir, images=getattr(self, im_name)
                     ),
                 )
@@ -49,23 +53,58 @@ class Study(skrt.core.Archive):
         #  dtype='RtDose',
         #  subdir='RTDOSE',
         #  exclude=['MVCT', 'CT'],
-        #  images=self.ct_scans
+        #  images=self.ct_images    
         #  )
 
         # Load CT-specific RT doses
         #  self.ct_doses = self.get_plan_data(
-        #  dtype='RtDose', subdir='RTDOSE/CT', images=self.ct_scans
+        #  dtype='RtDose', subdir='RTDOSE/CT', images=self.ct_images    
         #  )
-        #  self.ct_doses = self.correct_dose_scan_position(self.ct_doses)
+        #  self.ct_doses = self.correct_dose_image_position(self.ct_doses)
 
-    def correct_dose_scan_position(self, doses=[]):
-        """Correct for scan positions from CheckTomo being offset by one slice
-        relative to scan positions."""
+    def add_image(self, im, image_type="CT"):
+        '''Add a new image of a given image type.'''
+
+        # Ensure we have a list of this type of image
+        im_name = f"{image_type.lower()}_images"
+        if not hasattr(self, im_name):
+            setattr(self, im_name, [])
+        if image_type not in self.image_types:
+            self.image_types.append(image_type)
+
+        # Add image to the list
+        images = getattr(self, im_name)
+        if isinstance(im, Image):
+            im_to_add = im
+        else:
+            im_to_add = Image(im)
+        images.append(im)
+
+        # Ensure image has a timestamp
+        if im.timestamp == "None_None":
+            im.timestamp = skrt.core.generate_timestamp()
+
+        # Ensure corresponding structure list exists
+        struct_subdir = f"RTSTRUCT/{image_type}"
+        struct_name = f"{image_type.lower()}_structure_sets"
+        if not hasattr(self, struct_name):
+            setattr(self, struct_name, [])
+
+        # Add the image's structure sets to structure set list
+        structure_sets = getattr(self, struct_name)
+        for structure_set in im.get_structure_sets():
+            structure_sets.append(structure_set)
+            if structure_set.timestamp == "None_None":
+                structure_set.timestamp = skrt.core.generate_timestamp()
+
+    def correct_dose_image_position(self, doses=[]):
+        """Correct for image positions from CheckTomo being offset by one slice
+        relative to image positions."""
 
         for dose in doses:
             dx, dy, dz = dose.voxel_size
-            x0, y0, z0 = dose.scan_position
-            dose.scan_position = (x0, y0, z0 + dz)
+            x0, y0, z0 = dose.image_position
+            dose.image_position = (x0, y0, z0 + dz)
         return doses
 
     def get_machine_sublist(self, dtype="", machine="", ignore_case=True):
@@ -91,18 +130,18 @@ class Study(skrt.core.Archive):
         return sublist
 
     def get_mvct_selection(self, mvct_dict={}, min_delta_hours=0.0):
-        """Get a selection of MVCT scans which were taken at least
+        """Get a selection of MVCT images which were taken at least
         <min_delta_hours> apart. <mvct_dict> is a dict where the keys are
-        patient IDs, and the paths are directory paths from which to load scans
+        patient IDs, and the paths are directory paths from which to load images
         for that patient."""
 
-        # Find scans meeting the time separation requirement
+        # Find images meeting the time separation requirement
         if min_delta_hours > 0:
-            mvct_scans = get_time_separated_objects(self.mvct_scans, min_delta_hours)
+            mvct_images = get_time_separated_objects(self.mvct_images, min_delta_hours)
         else:
-            mvct_scans = self.mvct_scans
+            mvct_images = self.mvct_images
 
-        # Find scans matching the directory requirement
+        # Find images matching the directory requirement
         selected = []
         patient_id = self.get_patient_id()
         if patient_id in mvct_dict:
@@ -110,15 +149,15 @@ class Study(skrt.core.Archive):
             # Get all valid directories for this patient
             valid_dirs = [skrt.core.fullpath(path) for path in mvct_dict[patient_id]]
 
-            # Check for scans matching that directory requirement
-            for mvct in mvct_scans:
+            # Check for images matching that directory requirement
+            for mvct in mvct_images:
                 mvct_dir = os.path.dirname(mvct.files[-1].path)
                 if skrt.core.fullpath(mvct_dir) in valid_dirs:
                     selected.append(mvct)
 
-        # Otherwise, just return all scans for this patient
+        # Otherwise, just return all images for this patient
         else:
-            selection = mvct_scans
+            selection = mvct_images
 
         return selection
 
@@ -134,8 +173,8 @@ class Study(skrt.core.Archive):
 
         Subdirectories with names in <exclude> will be ignored.
 
-        Each dose-like object will be matched by timestamp to one of the scans
-        in <scans> (which should be a list of DatedStores), if provided."""
+        Each dose-like object will be matched by timestamp to one of the images
+        in <images> (which should be a list of DatedStores), if provided."""
 
         doses = []
 
@@ -188,13 +227,13 @@ class Study(skrt.core.Archive):
             new_doses = []
             for dose in doses:
 
-                # Search for scans with matching timestamp
+                # Search for images with matching timestamp
                 timestamp = os.path.basename(os.path.dirname(dose.path))
                 if images:
                     try:
                         dose.date, dose.time = timestamp.split("_")
-                        scan = get_dated_obj(images, dose)
-                        dose.machine = scan.machine
+                        image = get_dated_obj(images, dose)
+                        dose.machine = image.machine
                     except BaseException:
                         image = images[-1]
                         dose.date = image.date
@@ -206,7 +245,7 @@ class Study(skrt.core.Archive):
                 dose.couch_translation, dose.couch_rotation = get_couch_shift(dose.path)
                 # WARNING!
                 #     Couch translation third component (y) inverted with
-                #     respect to CT scan
+                #     respect to CT image
                 # WARNING!
                 new_doses.append(dose)
             doses = new_doses
@@ -268,19 +307,19 @@ class Study(skrt.core.Archive):
 
         return plan_dose
 
-    def get_structs(self, subdir="", images=[]):
-        """Make list of RtStruct objects found within a given subdir, and
-        set their associated scan objects."""
+    def get_structure_sets(self, subdir="", images=[]):
+        """Make list of StructureSet objects found within a given subdir and
+        set their associated image objects."""
 
-        # Find RtStruct directories associated with each scan
+        # Find structure set directories associated with each image
         groups = self.get_dated_objects(dtype=skrt.core.Archive, subdir=subdir)
 
-        # Load RtStruct files for each
-        structs = []
+        # Load structure set files for each
+        structure_sets = []
         for group in groups:
 
             # Find the matching Image for this group
-            image = Image(None, load=False)
+            image = None
             image_dir = os.path.basename(group.path)
             image_found = False
 
@@ -298,19 +337,20 @@ class Study(skrt.core.Archive):
                         image = im
                         break
 
-            # Find all RtStruct files inside the dir
+            # Find all structure set files inside the dir
             for file in group.files:
 
-                # Create RtStruct
-                rt_struct = RtStruct(file.path, image=image)
+                # Create StructureSet
+                structure_set = StructureSet(file.path, image=image)
 
                 # Add to Image
-                image.add_structs(rt_struct)
+                if image is not None:
+                    image.add_structure_set(structure_set)
 
                 # Add to list of all structure sets
-                structs.append(rt_struct)
+                structure_sets.append(structure_set)
 
-        return structs
+        return structure_sets
 
     def get_description(self):
         """Load a study description."""
@@ -318,12 +358,12 @@ class Study(skrt.core.Archive):
         # Find an object from which to extract description
         obj = None
         if self.studies:
-            obj = getattr(self, f"{self.im_types[0].lower()}_scans")[-1]
+            obj = getattr(self, f"{self.image_types[0].lower()}_images")[-1]
         description = ""
         if obj:
             if obj.files:
-                scan_path = obj.files[-1].path
-                ds = pydicom.read_file(fp=scan_path, force=True)
+                image_path = obj.files[-1].path
+                ds = pydicom.read_file(fp=image_path, force=True)
                 if hasattr(ds, "StudyDescription"):
                     description = ds.StudyDescription
 
@@ -343,7 +383,7 @@ class Study(skrt.core.Archive):
             plan_dose.time = dose.time
             plan_dose.timestamp = dose.timestamp
             plan_dose.summationType = "PLAN"
-            plan_dose.scanPosition = dose.scanPosition
+            plan_dose.imagePosition = dose.imagePosition
             plan_dose.reverse = dose.reverse
             plan_dose.voxelSize = dose.voxelSize
             plan_dose.transform_ijk_to_xyz = dose.transform_ijk_to_xyz
@@ -378,6 +418,23 @@ class Patient(skrt.core.PathData):
                                 self.get_dated_objects(dtype=Study, subdir=subdir)
                             )
 
+    def add_study(self, subdir='', timestamp=None, images=None, 
+                  image_type="CT"):
+        '''Add a new study.'''
+    
+        # Create empty Study object
+        s = Study("")
+        s.subdir = subdir
+        s.timestamp = timestamp if timestamp else skrt.core.generate_timestamp()
+
+        # Add images
+        if images:
+            for im in images:
+                s.add_image(im, image_type)
+
+        # Add to studies list
+        self.studies.append(s)
+
     def combined_files(self, attr, min_date=None, max_date=None):
         """Get list of all files of a given data type <attr> associated with
         this patient, within a given date range if specified."""
@@ -434,7 +491,7 @@ class Patient(skrt.core.PathData):
         obj = None
         if self.studies:
             obj = getattr(
-                self.studies[0], f"{self.studies[0].im_types[0].lower()}_scans"
+                self.studies[0], f"{self.studies[0].image_types[0].lower()}_images"
             )[-1]
 
         # Read demographic info from the object
@@ -499,10 +556,11 @@ class Patient(skrt.core.PathData):
     def write(
         self,
         outdir=".",
-        ext=".nii.gz",
+        ext=".dcm",
         to_ignore=None,
         overwrite=True,
-        structure_set=None,
+        structure_set="all",
+        root_uid=None
     ):
         """Write files tree."""
 
@@ -523,28 +581,30 @@ class Patient(skrt.core.PathData):
 
             # Make study directory
             study_dir = os.path.join(
-                patient_dir, os.path.relpath(study.path, self.path)
+                patient_dir, study.subdir, study.timestamp
             )
             if not os.path.exists(study_dir):
                 os.makedirs(study_dir)
 
             # Loop through image types
-            for im_type in study.im_types:
+            for image_type in study.image_types:
 
-                if im_type in to_ignore:
+                if image_type in to_ignore:
                     continue
 
-                im_type_dir = os.path.join(study_dir, im_type)
-                if not os.path.exists(im_type_dir):
-                    os.mkdir(im_type_dir)
+                image_type_dir = os.path.join(study_dir, image_type)
+                if not os.path.exists(image_type_dir):
+                    os.mkdir(image_type_dir)
 
-                # Write all scans of this image type
-                for im in getattr(study, f"{im_type.lower()}_scans"):
+                # Write all images of this image type
+                for im in getattr(study, f"{image_type.lower()}_images"):
 
-                    # Make directory for this scan
-                    im_dir = os.path.join(
-                        study_dir, os.path.relpath(im.path, study.path)
-                    )
+                    # Make directory for this image
+                    if im.path:
+                        im_timestamp = os.path.basename(im.path)
+                    else:
+                        im_timestamp = im.timestamp
+                    im_dir = os.path.join(image_type_dir, im_timestamp)
 
                     # Write image data to nifti
                     if ext == ".dcm":
@@ -553,17 +613,18 @@ class Patient(skrt.core.PathData):
                         outname = f"{im_dir}{ext}"
                     if os.path.exists(outname) and not overwrite:
                         continue
-                    im.write(outname)
+                    Image.write(im, outname, patient_id=self.id,
+                                modality=image_type, root_uid=root_uid)
 
                     # Find structure sets to write
                     if structure_set == "all":
-                        ss_to_write = im.structs
+                        ss_to_write = im.get_structure_sets()
                     elif structure_set is None:
                         ss_to_write = []
                     elif isinstance(structure_set, int):
-                        ss_to_write = [im.structs[structure_set]]
+                        ss_to_write = [im.get_structure_sets()[structure_set]]
                     elif skrt.core.is_list(structure_set):
-                        ss_to_write = [im.structs[i] for i in structure_set]
+                        ss_to_write = [im.get_structure_sets()[i] for i in structure_set]
                     else:
                         raise TypeError(
                             "Unrecognised structure_set option " f"{structure_set}"
@@ -574,12 +635,13 @@ class Patient(skrt.core.PathData):
 
                         # Find path to output structure directory
                         ss_path = os.path.join(
-                            study_dir, os.path.relpath(ss.path, study.path)
+                            study_dir, 'RTSTRUCT', image_type, im_timestamp
                         )
+                        filename = f'RTSTRUCT_{ss.timestamp}'
                         if ext == ".dcm":
-                            ss_dir = os.path.dirname(ss_path)
+                            ss_dir = ss_path
                         else:
-                            ss_dir = ss_path.replace(".dcm", "")
+                            ss_dir = os.path.join(ss_path, filename)
 
                         # Ensure it exists
                         if not os.path.exists(ss_path):
@@ -589,8 +651,9 @@ class Patient(skrt.core.PathData):
                         if ext == ".dcm":
                             if os.path.exists(ss_path) and not overwrite:
                                 continue
-                            ss.write(ss_path)
+                            ss.write(outname=f'{filename}.dcm', outdir=ss_path)
 
-                        # Write structs to individual files
+                        # Write ROIs to individual files
                         else:
                             ss.write(outdir=ss_dir, ext=ext)
+
